// Enemy 1 
// Cian Rice

// Engine Includes
#include "LogManager.h"
#include "WorldManager.h"
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "Enemy1.h"

Enemy1::Enemy1(Position start_pos) {

  WorldManager &wm = WorldManager::getInstance();
  wm.registerInterest(this, STEP_EVENT);

  setPosition(start_pos);
  
  ResourceManager &rm = ResourceManager::getInstance();

  Sprite *p_temp_sprite = rm.getSprite("enemy1");

  LogManager &lm = LogManager::getInstance();

  if (!p_temp_sprite) {
    lm.writeLog("Enemy1::Enemy1(): Warning! Sprite '%s' not found", 
			 "enemy1");
    return;
  }

  setSprite(p_temp_sprite, true);
  setSpriteSlowdown(3);

  setType("Enemy1"); // Set the enemy's type to the name 'Enemy 1'
  
  x_velocity = .25;

}

Enemy1::~Enemy1() {

  WorldManager &wm = WorldManager::getInstance();
  wm.unregisterInterest(this, STEP_EVENT);

}

int Enemy1::eventHandler(Event *e) {
  
  if (e->getType() == STEP_EVENT)
  {
	hitWaypoint();
	return 1;
  }
  
  return 0;
}

// Checks to see if its next to a waypoint
// If it is then it reverses direction
// It does this by iterating through a list of all game objects (not the best way, but due to limitations of the provided engine probably the only way)

void Enemy1::hitWaypoint() {

	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	GameObjectList obj_list = wm.getAllObjects();
	GameObjectListIterator obj_iterate = obj_list.createIterator();
	
	obj_iterate.first();
	while (!obj_iterate.isDone())
	{
		GameObject *current_obj = obj_iterate.currentObject();
		if (current_obj->getType() == "waypoint")
		{
			if (pos.getX() + 1 == current_obj->getPosition().getX())
			{
				setXVelocity(-x_velocity);	
				pos.setX(pos.getX() - 1);
			}
			else if (pos.getX() - 1 == current_obj->getPosition().getX())
			{
				setXVelocity(-x_velocity);	
				pos.setX(pos.getX() + 1);
			}
		}
		
		obj_iterate.next();
	}
}

