// Sonya Chen
//
// hero.cpp
//

#include "Hero.h"

#include "Sword.h"
#include "GameOver.h"
#include "Victory.h"

#include "LogManager.h"
#include "WorldManager.h"
#include "InputManager.h"
#include "GraphicsManager.h"
#include "ResourceManager.h"
#include "GameManager.h"

// Events
#include "EventKeyboard.h"
#include "EventStep.h"
#include "EventScore.h"
#include "EventOut.h"
#include "EventLife.h"

// Level Manager
#include "LevelManager.h"



Hero::Hero(Position start_pos){
  // Link to "hero" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite;
  p_temp_sprite = rm.getSprite("hero");
  if(!p_temp_sprite){
    lm.writeLog("Hero::Hero(): Warning! Sprite '%s' not found", "hero");
    return;
  }
  setSprite(p_temp_sprite, true);
  setSpriteSlowdown(0);
  
  
  InputManager &im = InputManager::getInstance();
  im.registerInterest(this, KEYBOARD_EVENT);
    
  setType("Hero");
  setSolid(true);
  
  WorldManager &wm = WorldManager::getInstance();
  wm.registerInterest(this, STEP_EVENT);
  wm.registerInterest(this, SCORE_EVENT);
  wm.registerInterest(this, LIFE_EVENT);
  
  setPosition(start_pos);
  wm.setViewFollowing(this);

  // Set values
  has_key = false;
  double_jump = false;
  can_double_jump = false; // Allows player to double jump 
                           //if haven't already and if have the ability
  
  lm.writeLog("Has Key? %d", has_key);
  height = 0;

  // HUD DISPLAY
  lives = 3;

  victory = 0;
  side = 1;
  move_right = false;
  move_left = false;
  start_position = start_pos;
}

Hero::~Hero(){
  InputManager &im = InputManager::getInstance();
  im.unregisterInterest(this,KEYBOARD_EVENT);

  WorldManager &wm = WorldManager::getInstance();
  wm.unregisterInterest(this, STEP_EVENT);
  wm.unregisterInterest(this, SCORE_EVENT);
  wm.unregisterInterest(this, LIFE_EVENT);
  
  // GameManager &game_manager = GameManager::getInstance();
  // game_manager.setGameOver();

  if (lives == 0)
    new GameOver("gameover");

  if (victory == 1)
    new Victory("victory");
}


int Hero::eventHandler(Event *e){
  WorldManager &wm = WorldManager::getInstance();
  LogManager &lm = LogManager::getInstance();


  if(e->getType() == KEYBOARD_EVENT){
    EventKeyboard *p_ke = static_cast <EventKeyboard *> (e);
    kbd(p_ke);
    return 1;
  }

  if(e->getType() == OUT_EVENT){
    death();
    return 1;
  }
  
  if (e->getType() == COLLISION_EVENT) {
    EventCollision *p_collide = static_cast <EventCollision *> (e);
    collision(p_collide);
  }

  if (e->getType() == LIFE_EVENT){
    death();
    return 1;
  }


  if (e->getType() == STEP_EVENT)
    {
      // Hero jump
      if(height > 0) // When height is greater than 0, jump up, and decrease height.
	{
	  move(0, -1);
	  height -= 1;
	}
      if(height == 0) // When height reaches 0, have hero drop down
	{
	  move(0, 1); 
	}
      
      move();
      return 1;
    }
  return 0;
}

// call move (or do nothing) according to key pressed
void Hero::kbd(EventKeyboard *p_ke){
  WorldManager &wm = WorldManager::getInstance();
  Position check_this = Position();
  GameObject *collided_with;   
  LogManager &lm = LogManager::getInstance();
  ResourceManager &rm = ResourceManager::getInstance();


  switch(p_ke -> getKey()){
  case KEY_LEFT:    // Left arrow
    side = -1;
    move_right = false;
    move_left = true;
    //    move(-1, 0);
    break;
  case KEY_RIGHT:   // Right arrow
    side = 1;
    move_right = true;
    move_left = false;
    //    move(+1, 0);
    break;
  case KEY_DOWN:
    move_right = false;
    move_left = false;
    break;


  case KEY_UP:      // Up arrow
    // Checks to see if hero is on the floor
    check_this.setX(pos.getX());
    check_this.setY(pos.getY() + 1);
    collided_with = wm.isCollision(this, check_this);

    // Double jump
    if(double_jump){
      if(can_double_jump){
	lm.writeLog("Did we get here?");
	if(collided_with == NULL){
	  height = 10;
	  lm.writeLog("Double Jump!");
	  can_double_jump = false;
	
	}
	else if (collided_with != NULL) {
	  if((collided_with->getType() == "Block") ||
	     (collided_with->getType() == "Platform")){   // if hero is on floor, can jump
	    height = 10;    // Set how high hero can jump
	    can_double_jump = true;
	    lm.writeLog("Single Jump?");
	  }
	}
      }	
	  
      else if (collided_with != NULL) {
	if((collided_with->getType() == "Block") ||
	   (collided_with->getType() == "Platform")){   // if hero is on floor, can jump
	  height = 10;    // Set how high hero can jump
	  can_double_jump = true;
	  lm.writeLog("Other Single Jump?");
	}
      }    
      else if(collided_with == NULL){ }
    }
    // Don't have double jump ability
    else{
      if(collided_with == NULL){ }
	  
      else if((collided_with->getType() == "Block") || 
	      (collided_with->getType() == "Platform"))   // if hero is on floor, can jump
	height = 10;    // Set how high hero can jump
    }

    break;
  case 32:          // Attack with sword (Spacebar)
    attack();
    break;
  };
  return;
}

// move hero
void Hero::move(int dx, int dy){
  WorldManager &wm = WorldManager::getInstance();
  Position new_pos(pos.getX() + dx, pos.getY() + dy);
  wm.moveObject(this, new_pos);
}

void Hero::move(){
  if(move_right){
    if(pos.getX() < LVL_MAX_X - 3){
      setXVelocity(1.2); // Move in the right direction
      setSpriteSlowdown(2);
    }
  }

  else if(move_left){
    if(pos.getX() > 2){
      setXVelocity(-1.2); // Move in the left direction
      setSpriteSlowdown(2);
    }
  }
  else {
    setSpriteSlowdown(0);
    setXVelocity(0); // Stop the sprite from moving
  }
	 
}


// When hero leaves screen
void Hero::death(){
  WorldManager &wm = WorldManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  //wm.markForDelete(this);
   
  lm.writeLog("LIVES: %d", lives);
  add_life(-1);
  setLives(-1);
  if(lives <= 0)
    wm.markForDelete(this);
   
  move_right = false;
  move_left = false;
   
   
  LevelManager &lvlm = LevelManager::getInstance();
   
  pos.setX(lvlm.new_hero_pos.getX());
  pos.setY(lvlm.new_hero_pos.getY());
  wm.setViewFollowing(this);
}


// Hero attacks with sword
void Hero::attack(){
  new Sword(pos, side);
}

// Add Life
inline void Hero::add_life(int life){
  lives += life;
}

// Event Handling for Different Collisions by Cian
// Based upon the saucer Collision code
void Hero::collision(EventCollision *p_collide) {
  
  WorldManager &wm = WorldManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  
  if (((p_collide->getObject1()->getType()) == "Key") ||
      ((p_collide->getObject2()->getType()) == "Key"))
    {
      has_key = true;
      lm.writeLog("Has Key? %d", has_key);
      
      if ((p_collide->getObject1()->getType()) == "Key")
	{
	  wm.markForDelete(p_collide -> getObject1());
	}
      else if ((p_collide->getObject2()->getType()) == "Key")
	{
	  wm.markForDelete(p_collide -> getObject2());
	}
      
    }
  else if (((p_collide->getObject1()->getType()) == "Enemy1") ||
	   ((p_collide->getObject2()->getType()) == "Enemy1"))
    {
      //EventLife life;
      //wm.onEvent(&life);
      death();
    }
  else if (((p_collide->getObject1()->getType()) == "Door") ||
	   ((p_collide->getObject2()->getType()) == "Door"))
    {
      lm.writeLog("Door Hit!");
      if (has_key == true)
	{
	  LevelManager &lvlm = LevelManager::getInstance();
	  if (lvlm.current_level < MAX_LEVEL)
	    {
	      lvlm.clearLevel(); // Clear the current level
	      has_key = false; // Remove the key
	      lvlm.current_level += 1; // Increment the level
	      lvlm.createLevels(); // Create the new level
	      setPosition(lvlm.new_hero_pos); // repositon the hero
	      
	      wm.setViewFollowing(this); // Reset the camera
	      
	      move_right = false;
	      move_left = false;
	    }
	  // If the player is at the last level, delete the player (game won!) 
	  else if (lvlm.current_level >= MAX_LEVEL) {
	    
	    if ((p_collide->getObject1()->getType()) == "Hero")
	      {
		victory = 1;
		wm.markForDelete(p_collide -> getObject1());
	      }
	    else if ((p_collide->getObject2()->getType()) == "Hero")
	      {
		victory = 1;
		wm.markForDelete(p_collide -> getObject2());
	      }
	  }
	}
    }
  // If the player hits a score item, then remove it and increment the score
  else if (((p_collide->getObject1()->getType()) == "Score Item") ||
	   ((p_collide->getObject2()->getType()) == "Score Item"))
    {
      setScore(100);
	  
      if ((p_collide->getObject1()->getType()) == "Score Item")
	{
	  wm.markForDelete(p_collide -> getObject1());
	}
      else if ((p_collide->getObject2()->getType()) == "Score Item")
	{
	  wm.markForDelete(p_collide -> getObject2());
	}
    }  

  // If the player hits a health pack, then remove it and increase lives
  else if (((p_collide->getObject1()->getType()) == "Health Item") ||
	   ((p_collide->getObject2()->getType()) == "Health Item")){
    add_life(1);
    setLives(1);

    if((p_collide->getObject1()->getType()) == "Health Item")
      wm.markForDelete(p_collide -> getObject1());
    else if((p_collide ->getObject2()->getType()) == "Health Item")
      wm.markForDelete(p_collide -> getObject2());
  }

  // If player hits powerup, then remove it and allow to double jump
  else if (((p_collide->getObject1()->getType()) == "Powerup Item") ||
	   ((p_collide->getObject2()->getType()) == "Powerup Item")){

    double_jump = true;

    if((p_collide->getObject1()->getType()) == "Powerup Item")
      wm.markForDelete(p_collide -> getObject1());
    else if((p_collide ->getObject2()->getType()) == "Powerup Item")
      wm.markForDelete(p_collide -> getObject2());
  }
}

void Hero::setScore(int point_count) {
  score_HUD->addPoints(point_count);
}

Score* Hero::setScoreHUD(Score* new_score_HUD) {
  score_HUD = new_score_HUD;
}

void Hero::setLives(int life_count) {
  lives_HUD->addLife(life_count);
}

Lives* Hero::setLivesHUD(Lives* new_lives_HUD) {
  lives_HUD = new_lives_HUD;
}

int Hero::getHeroLives() {
  return lives;
}
