// Platform
// Cian Rice

// Engine Includes
#include "LogManager.h"
#include "WorldManager.h"
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "Platform.h"

Platform::Platform(Position start_pos) {

  WorldManager &wm = WorldManager::getInstance();
  wm.registerInterest(this, STEP_EVENT);

  setPosition(start_pos);
  
  ResourceManager &rm = ResourceManager::getInstance();

  Sprite *p_temp_sprite = rm.getSprite("platform");

  LogManager &lm = LogManager::getInstance();

  if (!p_temp_sprite) {
    lm.writeLog("Platform::Platform(): Warning! Sprite '%s' not found", 
			 "platform");
    return;
  }

  setSprite(p_temp_sprite, true);
  setCentered(true);
  setSpriteSlowdown(3);

  setType("Platform"); // Set the type to platform
  
  x_velocity = .25;}

Platform::~Platform() {

  WorldManager &wm = WorldManager::getInstance();
  wm.unregisterInterest(this, STEP_EVENT);

}

int Platform::eventHandler(Event *e) {
  
  if (e->getType() == STEP_EVENT)
  {
	hitWaypoint();
	return 1;
  }
  
  return 0;
}

// This is similar to the enemy code in that the platforms checks to see if its next to a waypoint
// If it is then it reverses direction
// It does this by iterating through a list of all game objects (not the best way, but due to limitations of the provided engine probably the only way)

void Platform::hitWaypoint() {

	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	GameObjectList obj_list = wm.getAllObjects();
	GameObjectListIterator obj_iterate = obj_list.createIterator();
	
	obj_iterate.first();
	while (!obj_iterate.isDone())
	{
		GameObject *current_obj = obj_iterate.currentObject();
		if (current_obj->getType() == "waypoint")
		{
			if (pos.getX() + 1 == current_obj->getPosition().getX())
			{
				setXVelocity(-x_velocity);	
				pos.setX(pos.getX() - 1);
			}
			else if (pos.getX() - 1 == current_obj->getPosition().getX())
			{
				setXVelocity(-x_velocity);	
				pos.setX(pos.getX() + 1);
			}
		}
		
		obj_iterate.next();
	}
}

