// Sonya Chen
// 
// Sword.cpp
//

#include "Sword.h"

#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "Enemy1.h"

Sword::Sword(Position hero_pos, int side){

  WorldManager &wm = WorldManager::getInstance();
  wm.registerInterest(this, STEP_EVENT);

  // link to "sword" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  Sprite *p_temp_sprite;
  if (side == 1){ // Attacking to the right
    pos.setX(hero_pos.getX() + 3);
    pos.setY(hero_pos.getY());
    p_temp_sprite = rm.getSprite("sword_right");
  }
  else if(side == -1){ // Attacking to the left
    pos.setX(hero_pos.getX() - 2);
    pos.setY(hero_pos.getY());
    p_temp_sprite = rm.getSprite("sword_left");
  }


  if(!p_temp_sprite){
    LogManager &lm = LogManager::getInstance();
    lm.writeLog("Sword::Sword(): Warning! Sprite '%s' wasn't found", "sword");
    return;
  }

  setSprite(p_temp_sprite);
  setSpriteSlowdown(5);

  setType("Sword");


  //time_to_live = p_sprite->getFrameCount();
  time_to_live = 15;
}

Sword::~Sword(){
  WorldManager &wm = WorldManager::getInstance();
  wm.unregisterInterest(this, STEP_EVENT);
}

// Handle events
int Sword::eventHandler(Event *e){
  if(e->getType() == STEP_EVENT){
    step();
    return 1;
  }

  if(e->getType() == COLLISION_EVENT){
    EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    hit(p_collision_event);
    return 1;
  }

  return 0;
}


void Sword::step(){
  time_to_live--;
  if(time_to_live <= 0){
    WorldManager &wm = WorldManager::getInstance();
    wm.markForDelete(this);
  }
}

// if sword hits Enemy1, mark Enemy1 and sword for deletion
void Sword::hit(EventCollision *p_c){
  WorldManager &wm = WorldManager::getInstance();
  if((p_c->getObject1()->getType() == "Enemy1") ||
     (p_c->getObject2()->getType() == "Enemy1")){
    wm.markForDelete(p_c->getObject1());
    wm.markForDelete(p_c->getObject2());
  }
}
