// Sonya Chen
//
// TextMessage.cpp
//

// Based off of the saucer shoot code

#include "TextMessage.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "EventStep.h"
#include "ResourceManager.h"

TextMessage::TextMessage(string message) {

  // link to "message" sprite
  ResourceManager &resource_manager = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite = resource_manager.getSprite(message);
  if (!p_temp_sprite) {
    lm.writeLog("TextMessage::TextMessage: Warning! Sprite '%s' not found", 
		message.c_str());
    return;
  }
  setSprite(p_temp_sprite);
  setSpriteSlowdown(15);  
  time_to_live = p_temp_sprite->getFrameCount() * 15;

  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  altitude = 2;// make in the forground

  setType("TextMessage");
  is_solid = false;
  
  // put in center of screen
  pos.setX(world_manager.getView().getCorner().getX() + 
	   world_manager.getView().getHorizontal()/2);
  pos.setY(world_manager.getView().getCorner().getY() + 
	   world_manager.getView().getVertical()/2);
}

// when done, unregister
TextMessage::~TextMessage() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle event
// return 0 if ignored, else 1
int TextMessage::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

// count down to end of message
void TextMessage::step() {
  time_to_live--;
  if (time_to_live <= 0) { 
    WorldManager &gw = WorldManager::getInstance();
    gw.markForDelete(this);
  }
}

