// Engine includes
#include "GameManager.h"
#include "LogManager.h"
#include "ResourceManager.h"
#include "WorldManager.h"

// Other Includes
#include "GameStart.h"



// Function prototypes
void loadResources();
int display_string(string a_str, Position world_pos);


int main(int argc, char *argv[]) {
    LogManager &log_manager = LogManager::getInstance();

    // Start up Game Manager
    GameManager &game_manager = GameManager::getInstance();
    if (game_manager.startUp(false, true))  {
      log_manager.writeLog("Error starting game manager!\n");
      game_manager.shutDown();
    }

    WorldManager &wm = WorldManager::getInstance();
    wm.getView();
    
    log_manager.writeLog("View Box = Corner X: %d, Corner Y: %d, width: %d, height %d:", wm.getView().getCorner().getX(), wm.getView().getCorner().getY(),
			 wm.getView().getHorizontal(), wm.getView().getVertical());
    
    // Load up the resources and populate the world
    loadResources();
    new GameStart("gamestart");

    game_manager.run();

    // Shut everything down
    game_manager.shutDown();
}

void loadResources()
{

	// Load all the sprites up
  ResourceManager &rm = ResourceManager::getInstance();
  rm.loadSprite("sprites/enemy1-spr.txt", "enemy1");
  rm.loadSprite("sprites/hero-spr.txt", "hero");
  rm.loadSprite("sprites/platform-spr.txt", "platform");
  rm.loadSprite("sprites/key-spr.txt", "key");
  rm.loadSprite("sprites/door-spr.txt", "door");
  rm.loadSprite("sprites/gameover-spr.txt", "gameover");
  rm.loadSprite("sprites/logo-spr.txt", "gamestart");
  rm.loadSprite("sprites/victory-spr.txt", "victory");
  rm.loadSprite("sprites/swordright-spr.txt", "sword_right");
  rm.loadSprite("sprites/swordleft-spr.txt", "sword_left");
}


