///
/// A "collision" event
///

#ifndef __EVENT_COLLISION_H__
#define __EVENT_COLLISION_H__

#include "Event.h"
#include "GameObject.h"

#define COLLISION_EVENT "collision"

class EventCollision : public Event {

 protected:
  Position pos;		   ///< Where collision occurred.
  GameObject *obj1;        ///< The object moving, causing the collision.
  GameObject *obj2;	   ///< The object being collided with.

 public:
  EventCollision();

  /// Create collision event between o1 and o2 at position p.
  /// Object o1 "caused" the collision by moving into object o2.
  EventCollision(GameObject *o1, GameObject *o2, Position p);

  /// Return object that caused collision.
  GameObject *getObject1();

  /// Set object that caused collision.
  void setObject1(GameObject *new_o1); 

  /// Return object that was collided with.
  GameObject *getObject2();

  /// Set object that was collided with.
  void setObject2(GameObject *new_o2); 

  /// Return the position of the collision.
  Position getPosition();

  /// Set the position of the collision.
  void setPosition(Position new_pos);
};

#endif /// __EVENT_COLLISION_H__
