///
/// The game manager
///

#ifndef __GAME_MANAGER_H__
#define __GAME_MANAGER_H__

#include <time.h>		///< For time_t

#include "Manager.h"

#define DRAGONFLY_VERSION 1.7

#define DEFAULT_FRAME_TIME 33	///< In milliseconds (33 ms == 30 fps).

class GameManager : public Manager {

 private:
  GameManager (GameManager const&);    ///< Don't allow copy.
  void operator=(GameManager const&);  ///< Don't allow assignment.
  GameManager();		       ///< Private since a singleton.

 protected:
  bool game_over;		///< True -> game loop should stop.
  int frame_time;		///< Target time per game loop, in millisec.

 public:
  ~GameManager();

  /// Get the singleton instance of the GameManager.
  static GameManager &getInstance();

  /// Startup all the GameManager services.
  int startUp();

  /// Startup all the GameManager services.
  /// append = true if add to log file (default false).
  /// flush = true if flush after each write (default false).
  int startUp(bool append, bool flush);

  /// Startup all the GameManager services.
  /// append = true if add to log file (default false).
  /// flush = true if flush after each write (default false).
  /// seed = random seed (default is seed with system time).
  int startUp(bool append, bool flush, time_t seed);

  /// Shut down the GameManager services, also terminating the program.
  void shutDown();

  /// Run the game loop.
  void run();

  /// Run the game loop.
  /// fr_time is time between frames (default FRAME_TIME, defined above).
  void run(int fr_time);

  /// Indicate the game is over, which will stop the game loop.
  void setGameOver();

  /// Return frame time.  
  //  The frame time is the target for each game loop, in milliseconds.
  int getFrameTime();
};

#endif // __GAME_MANAGER_H__
