///
/// A list of GameObjects
///

#ifndef __GAME_OBJECT_LIST_H__
#define __GAME_OBJECT_LIST_H__

#define MAX_GAME_OBJECTS 10000

#include "GameObject.h"
#include "GameObjectListIterator.h"

class GameObjectListIterator;

class GameObjectList {

 protected:
  int count;			      ///< Count of objects in list.
  GameObject *list[MAX_GAME_OBJECTS]; ///< Array of pointers to objects.

 public:
  friend class GameObjectListIterator; ///< Iterators can access.
  GameObjectListIterator createIterator() const; ///< Create an iterator.

  GameObjectList();
  ~GameObjectList();

  /// \brief Insert game object pointer in list.
  // Return 0 if ok, else -1.
  int insert(GameObject *p_go);

  /// \brief Remove game object pointer from list,
  // Return 0 if found, else -1.
  int remove(GameObject *p_go);

  /// Clear the list (setting count to 0).
  void clear();

  /// Return pointer to object with indicated id, NULL of not found.
  GameObject *find(int id);

  /// Return count of number of objects in list.
  int getCount(void);
};

#endif // __GAME_OBJECT_LIST_H__
