///
/// The graphics manager
///

#ifndef __GRAPHICS_MANAGER_H__
#define __GRAPHICS_MANAGER_H__

#ifdef CYGWIN
#include <ncurses/curses.h>
#else
#include <curses.h>
#endif

/// Suppots colors, defined by curses:
///   COLOR_BLACK ("black" in Sprite file)
///   COLOR_RED ("red" in Sprite file)
///   COLOR_GREEN ("green" in Sprite file)
///   COLOR_YELLOW ("yellow" in Sprite file)
///   COLOR_BLUE ("blue" in Sprite file)
///   COLOR_MAGENTA ("magenta" in Sprite file)
///   COLOR_CYAN ("cyan" in Sprite file)
///   COLOR_WHITE ("white" in Sprite file)

#define COLOR_DEFAULT COLOR_WHITE //< If color not specified, will use this.

#include "Manager.h"
#include "Position.h"
#include "Frame.h"

class GraphicsManager : public Manager {

 private:
  GraphicsManager (GraphicsManager const&); ///< Don't allow copy.
  void operator=(GraphicsManager const&);   ///< Don't allow assignment.
  GraphicsManager();		            ///< Private since a singleton.

 protected:
  WINDOW *win1, *win2;		///< Two window buffers for drawing.
  WINDOW *curr_win;		///< Current buffer.
  int window_horiz;		///< Max window width.
  int window_vert;		///< Max window height.

 public:
  ~GraphicsManager();

  /// Get the one and only instance of the GraphicsManager.
  static GraphicsManager &getInstance();

  /// Convert world position to screen position (based on the view).
  Position worldToScreen(Position world_pos);

  /// Get terminal ready for text-based display.
  /// Return 0 if ok, else return negative number.
  int startUp();

  /// Revert back to normal terminal display.
  void shutDown();

  /// Render current display buffer.
  /// Return 0 if ok, else -1.
  int swapBuffers();

  /// Put a character at screen location (x,y) (with default color).
  /// Note: top-left coordinate is (0.0).
  /// Return 0 if ok, else -1.
  int drawCh(Position world_pos, char ch);

  /// Put a character at screen location (x,y) with color.
  /// Note: top-left coordinate is (0.0).
  /// Return 0 if ok, else -1.
  int drawCh(Position world_pos, char ch, int color);

  /// Draw a single sprite frame at screen location (x,y) with color.
  /// If centered true thyen center frame centered at (x,y).
  /// Note: top-left coordinate is (0,0).
  /// Return 0 if ok, else -1.
  int drawFrame(Position world_pos, Frame frame, bool centered, int color);

  /// Draw a single sprite frame at screen location (x,y) (with
  /// default color).
  /// If centered true thyen center frame centered at (x,y).
  /// Note: top-left coordinate is (0,0).
  /// Return 0 if ok, else -1.
  int drawFrame(Position world_pos, Frame frame, bool centered);

  /// Return display's horizontal maximum.
  int getHorizontal();

  /// Return display's vertical maximum.
  int getVertical();

};

#endif //__GRAPHICS_MANAGER_H__

