///
/// The log manager
/// 

#ifndef __LOG_MANAGER_H__
#define __LOG_MANAGER_H__

#include "Manager.h"

#define LOGFILE_NAME "dragonfly.log"

class LogManager : public Manager {

 private:
  LogManager (LogManager const&);    ///< Don't allow copy.
  void operator=(LogManager const&); ///< Don't allow assignment.
  LogManager();			     ///< Private since a singleton.

 protected:
  bool do_flush;		     ///< True if fflush after each write.
  FILE *fp;			     ///< Pointer to log file.
  char *getTimeString();	     ///< Returns pretty-formatted string.

 public:
  ~LogManager();

  /// Get the one and only instance of the LogManager.
  static LogManager &getInstance();

  /// \brief Startup the LogManager (open logfile "dragonfly.h").
  /// append = true to add to logfile, flush = true if flush() afer each write.
  int startUp(bool append, bool flush);

  /// Shutdown the LogManager (close logfile).
  void shutDown();

  /// \brief Write to logfile.
  /// Supports printf() formatting of strings.
  /// Return number of bytes written, -1 if error.
  int writeLog(const char *fmt, ...);
};

#endif // __LOG_MANAGER_H__
