///
/// The base manager
///

#ifndef __MANAGER_H__
#define __MANAGER_H__

#include "GameObjectList.h"
#include "Event.h"

#define MAX_EVENTS 100		          ///< Max number of different events.

class Manager {

 protected:
  string event[MAX_EVENTS];               ///< Names of events.
  GameObjectList obj_list[MAX_EVENTS];    ///< Objects in list.
  int event_list_count;		          ///< Number of events in lists.
  bool is_started;			  ///< True when startUp successful.

  /// Check if event is allowed by the Manager.
  /// If allowed, return 1 else 0.
  virtual bool isValid(string event_name);
  
 public:
  Manager();
  virtual ~Manager();
  
  /// Send event to all interested objects.
  void onEvent(Event *p_event);

  /// Indicate interest in event.
  /// Return 0 if ok, else -1.
  /// Note, no check is made to see if the GameObject is already registered.
  int registerInterest(GameObject *p_go, string event_name);

  /// Indicate no longer interest in event. 
  /// Return 0 if ok, else -1.
  int unregisterInterest(GameObject *p_go, string event_name);

  /// Startup the Manager. 
  /// Return 0 if ok, else negative number.
  virtual int startUp();

  /// Shutdown the Manager.  
  virtual void shutDown();

  /// Return status of is_started (true when startUp() successful).
  bool isStarted();

};

#endif /// __MANAGER_H__


