///
/// The resource manager
///

#ifndef __RESOURCE_MANAGER_H__
#define __RESOURCE_MANAGER_H__

#define MAX_SPRITES 10000

#include <string>

using namespace std;

#include "Manager.h"
#include "Sprite.h"

// Define delimiters used to parse Sprite files -
// the ResourceManager "knows" the file format.
#define FRAMES_TOKEN "frames"
#define HEIGHT_TOKEN "height"
#define WIDTH_TOKEN "width"
#define COLOR_TOKEN "color"
#define END_FRAME_TOKEN "end"
#define END_SPRITE_TOKEN "eof"

class ResourceManager : public Manager {

 private:
  ResourceManager (ResourceManager const&); ///< Don't allow copy.
  void operator=(ResourceManager const&);   ///< Don't allow assignment.
  ResourceManager();		            ///< Private since a singleton.

 protected:
  Sprite *sprite[MAX_SPRITES];	///< Array of sprites.
  int sprite_count;		///< Count of number of loaded sprites.

 public:
  ~ResourceManager();

  /// Get the one and only instance of the ResourceManager.
  static ResourceManager &getInstance();

  /// Load Sprite from file.
  /// Assign the indicated label to sprite.
  /// Return 0 if ok, else -1.
  int loadSprite(string filename, string label);

  /// Find Sprite with indicated label.
  /// Return pointer to it if found, else NULL.
  Sprite *getSprite(string label);

  int startUp();
  void shutDown();
};

#endif //__RESOURCE_MANAGER_H__
